/*****************************************************************************
 *  Rohde&Schwarz DC power supplies instrument driver
 *  Original Release: May 2014
 *  By: Lukas Hynek
 *
 *  Should you have any technical questions please contact the hotline of
 *  Rohde & Schwarz Vertriebs-GmbH Rohde & Schwarz Support Center
 *
 *  e-mail: CustomerSupport@rohde-schwarz.com
 *
 *  Modification History:
 *    see ChangeLog
 *
 *****************************************************************************/
#include "rshmc804x.h"
#if (defined(WIN32) || defined(_WIN32) || defined(__WIN32__) || defined(__NT__))
#define ANSI
#endif
#ifndef ANSI                /* UNIX compatible */
    #include <sys/timeb.h>
    #include <sys/types.h>
    #include <unistd.h>
    #define Sleep sleep
#else                       /* ANSI compatible */
#include <windows.h>
#endif

/*****************************************************************************
 *- Value Definition and Range Tables ---------------------------------------*
 *****************************************************************************/
static ViString deviceArr[] = {"INT", "EXT", "DEF", NULL};
static ViString regGroupArr[] = {"OPER", "QUES", "QUES:INST", "QUES:INST:ISUM", NULL};
static ViString regTypeArr[] = {"COND", "EVEN", "ENAB", NULL};

/*****************************************************************************
 *- Callback Declarations ---------------------------------------------------*
 *****************************************************************************/

/*****************************************************************************
 *- User-Callable Functions (Exportable Functions) --------------------------*
 *****************************************************************************/

/*****************************************************************************
 * Function: rshmc804x_init
 * Purpose:  VXIplug&play required function. Calls the
 *           rshmc804x_InitWithOptions function.
 *****************************************************************************/
ViStatus _VI_FUNC rshmc804x_init(ViRsrc resourceName, ViBoolean IDQuery,
                                 ViBoolean resetDevice, ViSession* newInstrSession)
{
    ViStatus error = VI_SUCCESS;

    if (newInstrSession == NULL)
    {
        (void)RsCore_SetErrorInfo(0, VI_FALSE, RS_ERROR_INVALID_PARAMETER,
                                  VI_ERROR_PARAMETER4, "Null address for Instrument Handle");

        checkErr(RS_ERROR_INVALID_PARAMETER);
    }

    checkErr(rshmc804x_InitWithOptions (resourceName, IDQuery, resetDevice, "", newInstrSession));

Error:
    return error;
}

/*****************************************************************************
 * Function: rshmc804x_InitWithOptions
 * Purpose:  This function creates a new RS session and calls the
 *           RsInit function.
 *****************************************************************************/
ViStatus _VI_FUNC rshmc804x_InitWithOptions(
    ViRsrc resourceName,
    ViBoolean IDQuery,
    ViBoolean resetDevice,
    ViString optionString,
    ViSession* newInstrSession
)
{
    ViStatus error = VI_SUCCESS;
    ViSession instrSession = 0;
    RsCoreSessionPtr rsSession = NULL;
    ViChar completeOptionString[RS_MAX_MESSAGE_BUF_SIZE];
    ViPChar instrModel = NULL;

    if (newInstrSession == NULL)
    {
        (void)RsCore_SetErrorInfo(0, VI_FALSE, RS_ERROR_INVALID_PARAMETER, VI_ERROR_PARAMETER5,
                                  "Null address for Instrument Handle");
        checkErr(RS_ERROR_INVALID_PARAMETER);
    }

    *newInstrSession = 0;

    // TCPIP, USB and GPIB sessions to non-vxi capable
    (void)RsCore_CopyToUserBufferAsciiData(completeOptionString, RS_MAX_MESSAGE_BUF_SIZE, optionString);
    (void)RsCore_AppendToCsvString(completeOptionString, ",", RS_MAX_MESSAGE_BUF_SIZE, "AdditionalFlags=3");

    checkErr(RsCore_NewSpecificDriver(
        resourceName, // VISA Resource Name
        "rshmc804x", // driver prefix
        completeOptionString, // Init Options string, is applied to the session settings
        rshmc804x_RsCoreAttributeList, // List of all attributes from the rsxxx_AttrPropertiesList
        rshmc804x_RsCoreAttributeListLength, // Length of the attributes list
        VI_FALSE, //idnModelFullName Model has the full *IDN? query name e.g. true: "HMC8043", VI_FALSE: "HMC" Used in CheckInstrument()
        0, // WriteDelay
        0, // ReadDelay
        10000, // IO Segment Size
        RS_VAL_OPCWAIT_STBPOLLING, // OPC Wait Mode
        RSHMC804X_OPC_TIMEOUT, // OPC timeout
        10000, // VISA Timeout
        100000, // Self-test timeout
        RS_VAL_BIN_FLOAT_FORMAT_SINGLE_4BYTES, // BinaryFloatNumbersFormat
        RS_VAL_BIN_INTEGER_FORMAT_INT32_4BYTES, // binaryIntegerNumbersFormat
        &instrSession));

    // Minimal allowed rscore version is 4.6.0
    checkErr(RsCore_CheckMinimalEngineVersion(instrSession, 4, 6, 0));

    checkErr(RsCore_GetRsSession(instrSession, &rsSession));

    // No SCPI command has been sent yet.
    // Call viClear before sending any SCPI command
    checkErr(RsCore_ViClear(instrSession));

    // --- Here perform settings that are default for this driver, but can be overwritten by the optionsString settings
    rsSession->assureResponseEndWithLF = VI_TRUE;
    rsSession->opcSyncQueriesMechanism = RS_OPCSYNCQUERIES_ONLYCHECKMAVERRQUEUE;

    // Parse option string and set the desired values
    checkErr(RsCore_ApplyOptionString(instrSession, optionString));
        checkErr(RsCore_BuildRepCapTable(instrSession, rshmc804x_RsCoreRepCapTable, rshmc804x_RsCoreRepCapTableCount));

    // Query *IDN? string, parse it and set the following attributes:
    // - RS_ATTR_INSTRUMENT_MANUFACTURER
    // - RS_ATTR_INSTRUMENT_MODEL - based on the idnModelFullName true: "HMC8043", false : "HMC"
    // - RS_ATTR_INSTRUMENT_FIRMWARE_REVISION
    checkErr(RsCore_QueryAndParseIDNstring(instrSession, RSHMC804X_SIMULATION_ID_QUERY, NULL));

    // Not all instruments support *OPT? - only the instrument starting with NG...
    instrModel = (ViPChar)rsSession->attrInstrModelPtr->p2value;
    if (strlen(instrModel) >= 2 && strncmp(instrModel, "NG", 2) == 0)
    {
        checkErr(RsCore_QueryAndParseOPTstring(instrSession, RSHMC804X_SIMULATION_OPT_QUERY, RS_VAL_OPTIONS_PARSING_AUTO));
    }

    // Default Instrument Setup + optional *RST
    if (resetDevice == VI_TRUE)
    {
        checkErr(rshmc804x_reset(instrSession));
    }
    else
    {
        checkErr(rshmc804x_DefaultInstrSetup(instrSession));
    }

    if (IDQuery == VI_TRUE)
    {
        checkErr(RsCore_FitsIDNpattern(instrSession, RSHMC804X_VALID_ID_RESPONSE_STRING, NULL));
    }

    checkErr(rshmc804x_CheckStatus(instrSession));

Error:
    if (error < VI_SUCCESS)
    {
        if (error == RS_ERROR_INSTRUMENT_STATUS && rsSession->instrStatusStbQueryFirst == VI_TRUE && rsSession->autoSystErrQuery == VI_FALSE)
        {
            rsSession->autoSystErrQuery = VI_TRUE;
            (void)rshmc804x_CheckStatus(instrSession);
            rsSession->autoSystErrQuery = VI_FALSE;
        }

        if (instrSession != 0)
            (void)RsCore_ViClose(instrSession);
    }
    else
    {
        *newInstrSession = instrSession;
    }

    return error;
}

/*****************************************************************************
 * Function: rshmc804x_SetAttribute<type> Functions
 * Purpose:  These functions enable the instrument driver user to set
 *           attribute values directly.  There are typesafe versions for
 *           ViInt32, ViReal64, ViString, ViBoolean, and ViSession datatypes.
 *****************************************************************************/
/* ViInt32 */
ViStatus _VI_FUNC rshmc804x_SetAttributeViInt32(ViSession instrSession,
                                                ViString  repeatedCapabilityName,
                                                ViUInt32  attributeID,
                                                ViInt32   value)
{
    ViStatus error = VI_SUCCESS;

    (void)RsCore_LockSession(instrSession);

    error = RsCore_SetAttributeViInt32(instrSession, repeatedCapabilityName, attributeID, RS_VAL_DIRECT_USER_CALL, value);

    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/* ViReal64 */
ViStatus _VI_FUNC rshmc804x_SetAttributeViReal64(ViSession instrSession,
                                                 ViString  repeatedCapabilityName,
                                                 ViUInt32  attributeID,
                                                 ViReal64  attributeValue)
{
    ViStatus error = VI_SUCCESS;

    (void)RsCore_LockSession(instrSession);

    error = RsCore_SetAttributeViReal64(instrSession, repeatedCapabilityName, attributeID, RS_VAL_DIRECT_USER_CALL, attributeValue);

    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/* ViString */
ViStatus _VI_FUNC rshmc804x_SetAttributeViString(ViSession instrSession,
                                                 ViString  repeatedCapabilityName,
                                                 ViUInt32  attributeID,
                                                 ViString  attributeValue)
{
    ViStatus error = VI_SUCCESS;

    (void)RsCore_LockSession(instrSession);

    error = RsCore_SetAttributeViString(instrSession, repeatedCapabilityName, attributeID, RS_VAL_DIRECT_USER_CALL, attributeValue);

    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/* ViBoolean */
ViStatus _VI_FUNC rshmc804x_SetAttributeViBoolean(ViSession instrSession,
                                                  ViString  repeatedCapabilityName,
                                                  ViUInt32  attributeID,
                                                  ViBoolean attributeValue)
{
    ViStatus error = VI_SUCCESS;

    (void)RsCore_LockSession(instrSession);

    error = RsCore_SetAttributeViBoolean(instrSession, repeatedCapabilityName, attributeID, RS_VAL_DIRECT_USER_CALL, attributeValue);

    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*****************************************************************************
 * Function: rshmc804x_GetAttribute<type> Functions
 * Purpose:  These functions enable the instrument driver user to get
 *           attribute values directly.  There are typesafe versions for
 *           ViInt32, ViReal64, ViString, ViBoolean, and ViSession attributes.
 *****************************************************************************/

/* ViInt32 */
ViStatus _VI_FUNC rshmc804x_GetAttributeViInt32(ViSession instrSession,
                                                ViString  repeatedCapabilityName,
                                                ViUInt32  attributeID,
                                                ViInt32*  attributeValue)
{
    ViStatus error = VI_SUCCESS;

    (void)RsCore_LockSession(instrSession);
    error = RsCore_GetAttributeViInt32(instrSession, repeatedCapabilityName, attributeID, RS_VAL_DIRECT_USER_CALL, attributeValue);

    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/* ViReal64 */
ViStatus _VI_FUNC rshmc804x_GetAttributeViReal64(ViSession instrSession,
                                                 ViString  repeatedCapabilityName,
                                                 ViUInt32  attributeID,
                                                 ViReal64* attributeValue)
{
    ViStatus error = VI_SUCCESS;

    (void)RsCore_LockSession(instrSession);

    error = RsCore_GetAttributeViReal64(instrSession, repeatedCapabilityName, attributeID, RS_VAL_DIRECT_USER_CALL, attributeValue);

    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/* ViString */
ViStatus _VI_FUNC rshmc804x_GetAttributeViString(ViSession instrSession,
                                                 ViString  repeatedCapabilityName,
                                                 ViUInt32  attributeID,
                                                 ViInt32   bufferSize,
                                                 ViChar    attributeValue[])
{
    ViStatus error = VI_SUCCESS;

    (void)RsCore_LockSession(instrSession);

    error = RsCore_GetAttributeViString(instrSession, repeatedCapabilityName, attributeID, RS_VAL_DIRECT_USER_CALL, bufferSize,
        attributeValue);

    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/* ViBoolean */
ViStatus _VI_FUNC rshmc804x_GetAttributeViBoolean(ViSession  instrSession,
                                                  ViString   repeatedCapabilityName,
                                                  ViUInt32   attributeID,
                                                  ViBoolean* value)
{
    ViStatus error = VI_SUCCESS;

    (void)RsCore_LockSession(instrSession);

    error = RsCore_GetAttributeViBoolean(instrSession, repeatedCapabilityName, attributeID, RS_VAL_DIRECT_USER_CALL, value);

    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*****************************************************************************
 * Function: Rs_GetAttributeRepCapNameId
 * Purpose:  This function returns the repeated capability id(s) that
 *           belongs to the attribute you specify. More then one id is listed
 *           in comma separated string.
 *****************************************************************************/
ViStatus _VI_FUNC rshmc804x_GetAttributeRepeatedCapabilityIds(
                            ViSession instrSession,
                            ViUInt32  attributeID,
                            ViInt32   bufferSize,
                            ViChar    repeatedCapabilityIds[])
{
    ViStatus error = VI_SUCCESS;

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, bufferSize, 1, RS_MAX_SHORT_MESSAGE_BUF_SIZE),
            3, "Buffer Size");

    checkErr(RsCore_GetAttributeRepCapNameIds(instrSession, attributeID, bufferSize, repeatedCapabilityIds));

Error:
    return error;
}

/*****************************************************************************
 * Function: Get Attribute Repeated Capability Id Name(s)
 * Purpose:  This function returns the repeated capability name(s) that
 *           belongs to the attribute and single repeated capability id you
 *           may specify. More then one name is listed in comma separated
 *           string.
 *****************************************************************************/
ViStatus _VI_FUNC rshmc804x_GetAttributeRepeatedCapabilityIdNames(
                            ViSession instrSession,
                            ViUInt32  attributeID,
                            ViString  repeatedCapabilityId,
                            ViInt32   bufferSize,
                            ViChar    repeatedCapabilityIdNames[])
{
    ViStatus error = VI_SUCCESS;

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, bufferSize, 1, RS_MAX_SHORT_MESSAGE_BUF_SIZE),
            4, "Buffer Size");

    checkErr(RsCore_GetAttributeRepCapNamesAll(instrSession,
        attributeID,
        repeatedCapabilityId,
        bufferSize,
        repeatedCapabilityIdNames));

Error:
    return error;
}

/// HIFN Function used by Hi-level functions to obtain a repCap name from:
/// HIFN - attributeID
/// HIFN - repCapNameID
/// HIFN - 0-based index in the repCapNames
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rshmc804x_init or
/// HIPAR instrSession/rshmc804x_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR attributeId/Pass the ID of an attribute.
/// HIPAR repCapNameId/RepCap Name ID.
/// HIPAR index/Zero based index in the repCapNames.
/// HIPAR bufferSize/Pass the number of bytes in the string you allocate for the repCapName.
/// HIPAR repCapName/String returning RepCap name
ViStatus _VI_FUNC rshmc804x_GetAttributeRepCapName(ViSession instrSession,
                                                   ViUInt32  attributeId,
                                                   ViString  repCapNameId,
                                                   ViInt32   index,
                                                   ViInt32   bufferSize,
                                                   ViChar    repCapName[])
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, attributeId, repCapNameId, index, bufferSize, repCapName));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*****************************************************************************
 * Function: Reset
 * Purpose:  This function resets the instrument.
 *****************************************************************************/
ViStatus _VI_FUNC rshmc804x_reset(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    if (RsCore_Simulating(instrSession) == VI_FALSE)
    {
        checkErr(rshmc804x_ClearStatus(instrSession));
        checkErr(RsCore_Write(instrSession, "*RST")); // IGNORE: Check double write
        checkErr(RsCore_QueryViStringShort(instrSession, "*OPC?", NULL));
    }

    checkErr(rshmc804x_DefaultInstrSetup(instrSession));
    checkErr(rshmc804x_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);

    return error;
}

/*****************************************************************************
 * Function: Self Test
 * Purpose:  This function executes the instrument self-test and returns the
 *           result.
 *****************************************************************************/
ViStatus _VI_FUNC rshmc804x_self_test(ViSession instrSession,
                                      ViInt16*  testResult,
                                      ViChar    testMessage[])
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidNullPointer(instrSession, testResult), 2, "Null address for Test Result");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, testMessage), 3, "Null address for Test Message");

    checkErr(RsCore_SelfTest(instrSession, testResult, 48, testMessage));

    checkErr(rshmc804x_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*****************************************************************************
 * Function: Revision Query
 * Purpose:  This function returns the driver and instrument revisions.
 *****************************************************************************/
ViStatus _VI_FUNC rshmc804x_revision_query(ViSession instrSession,
                                           ViChar    instrumentDriverRevision[],
                                           ViChar    firmwareRevision[])
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidNullPointer(instrSession, instrumentDriverRevision), 2, "Null address for Instrument Driver Revision");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, firmwareRevision), 3, "Null address for Instrument Revision");

    checkErr(RsCore_GetAttributeViString(instrSession, NULL, RS_ATTR_SPECIFIC_DRIVER_REVISION, 0, 256, instrumentDriverRevision));
    checkErr(RsCore_GetAttributeViString(instrSession, NULL, RS_ATTR_INSTRUMENT_FIRMWARE_REVISION, 0, 256, firmwareRevision));
    checkErr(rshmc804x_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Disables or enables the beeper tone.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rshmc804x_init or
/// HIPAR instrSession/rshmc804x_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR beeperState/Disables or enables the beeper tone.
ViStatus _VI_FUNC rshmc804x_BeeperState(ViSession instrSession,
                                        ViBoolean beeperState)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rshmc804x_SetAttributeViBoolean(instrSession, "", RSHMC804X_ATTR_SYSTEM_BEEPER_ENABLED, beeperState),
            2, "Beeper State");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  When sending a command, the device emits a beep.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rshmc804x_init or
/// HIPAR instrSession/rshmc804x_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rshmc804x_StartBeep(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rshmc804x_SetAttributeViString(instrSession, "", RSHMC804X_ATTR_SYSTEM_BEEP_START, NULL));

    checkErr(rshmc804x_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Local operating mode (front panel control).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rshmc804x_init or
/// HIPAR instrSession/rshmc804x_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rshmc804x_SystemLocal(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rshmc804x_SetAttributeViString(instrSession, "", RSHMC804X_ATTR_SYSTEM_LOCAL, NULL));

    checkErr(rshmc804x_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Remote control (front panel control locked).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rshmc804x_init or
/// HIPAR instrSession/rshmc804x_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rshmc804x_SystemRemote(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rshmc804x_SetAttributeViString(instrSession, "", RSHMC804X_ATTR_SYSTEM_REMOTE, NULL));

    checkErr(rshmc804x_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  The front panel control is locked; with the SystemLocal method the front panel
/// HIFN  control will be activated.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rshmc804x_init or
/// HIPAR instrSession/rshmc804x_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rshmc804x_SystemRWLock(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rshmc804x_SetAttributeViString(instrSession, "", RSHMC804X_ATTR_SYSTEM_RWLOCK, NULL));

    checkErr(rshmc804x_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*****************************************************************************
 * Function:    System Version
 * Purpose:     This function queries the number of the SCPI version, which
 *              is relevant for the instrument.
 *****************************************************************************/
ViStatus _VI_FUNC rshmc804x_SystemVersion(ViSession instrSession,
                                          ViInt32   length,
                                          ViChar    systemVersion[])
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rshmc804x_GetAttributeViString(instrSession, "", RSHMC804X_ATTR_SYSTEM_VERSION,
            length, systemVersion),
            3, "System Version");
    checkErr(rshmc804x_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function returns the list of options installed on the instrument.
/// HIFN
/// HIFN     Note(s):
/// HIFN
/// HIFN     (1) This function is available only for NGA instrument
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rshmc804x_init or
/// HIPAR instrSession/rshmc804x_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR length/Sets the allocated memory length of System Options string.
/// HIPAR systemOptions/Returns the list of options installed on the instrument.
ViStatus _VI_FUNC rshmc804x_SystemOptions(ViSession instrSession,
                                          ViInt32   length,
                                          ViChar    systemOptions[])
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rshmc804x_GetAttributeViString(instrSession, "", RSHMC804X_ATTR_SYSTEM_OPTIONS,
            length, systemOptions),
            3, "System Options");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Returns device foot print info
/// HIFN
/// HIFN     Note(s):
/// HIFN
/// HIFN     (1) This function is available only for NGA instrument
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rshmc804x_init or
/// HIPAR instrSession/rshmc804x_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR length/Sets the allocated memory length of System Device Foot Print string.
/// HIPAR systemDeviceFootPrint/Returns device foot print info
ViStatus _VI_FUNC rshmc804x_SystemDeviceFootPrint(ViSession instrSession,
                                                  ViInt32   length,
                                                  ViChar    systemDeviceFootPrint[])
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rshmc804x_GetAttributeViString(instrSession, "", RSHMC804X_ATTR_SYSTEM_DEVICE_FOOT_PRINT,
            length, systemDeviceFootPrint),
            3, "System Device Foot Print");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*****************************************************************************
 * Function: Error Query
 * Purpose:  This function queries the instrument error queue and returns
 *           the result.
 *****************************************************************************/
ViStatus _VI_FUNC rshmc804x_error_query(ViSession instrSession,
                                        ViInt32*  errorCode,
                                        ViChar    errorMessage[])
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidNullPointer(instrSession, errorCode), 2, "Null address for Error Code");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, errorMessage), 3, "Null address for Error Message");

    checkErr(RsCore_ErrorQueryAll(instrSession, errorCode, 1024, errorMessage));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*****************************************************************************
 * Function: Error Message
 * Purpose:  This function translates the error codes returned by this
 *           instrument driver into user-readable strings.
 *
 *           Note:  The caller can pass NULL for the vi parameter.  This
 *           is useful if one of the init functions fail.
 *****************************************************************************/
ViStatus _VI_FUNC rshmc804x_error_message(ViSession instrSession,
                                          ViStatus  errorCode,
                                          ViChar    errorMessage[])
{
    return rshmc804x_utility_error_message(instrSession, errorCode, errorMessage);
}

/// HIFN  This function sets the Enable of selected registers.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rshmc804x_init or
/// HIPAR instrSession/rshmc804x_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR registerGroup/This control selects the registers that will be configured.
/// HIPAR ISummary/Selects the specific output.
/// HIPAR registerType/This control selects the registers that will be configured.
/// HIPAR registerValue/Sets content of selected register.
ViStatus _VI_FUNC rshmc804x_SetStatusRegister(ViSession instrSession,
                                              ViInt32   registerGroup,
                                              ViInt32   ISummary,
                                              ViInt32   registerType,
                                              ViInt32   registerValue)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, registerGroup, 0, 3),
            2, "Register Group");

    if (registerType != RSHMC804X_VAL_REGISTER_TYPE_ENAB)
    {
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, registerType),
            4, "Register Type");
    }

    if (registerGroup != RSHMC804X_VAL_REGISTER_GROUP_QUES_INST_ISUM)
    {
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "STAT:%s:%s %ld", regGroupArr[registerGroup], regTypeArr[registerType], registerValue);
    }
    else
    {
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "STAT:%s%ld:%s %ld", regGroupArr[registerGroup], ISummary, regTypeArr[registerType], registerValue);
    }
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rshmc804x_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function queries the status reporting system.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rshmc804x_init or
/// HIPAR instrSession/rshmc804x_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR registerGroup/This control selects the registers that will be configured.
/// HIPAR ISummary/Selects the specific output.
/// HIPAR registerType/This control selects the registers that will be configured.
/// HIPAR registerValue/Returns content of selected register.
ViStatus _VI_FUNC rshmc804x_GetStatusRegister(ViSession instrSession,
                                              ViInt32   registerGroup,
                                              ViInt32   ISummary,
                                              ViInt32   registerType,
                                              ViInt32*  registerValue)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, registerGroup, 0, 3),
            2, "Register Group");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, registerType, 0, 2),
            4, "Register Type");

    if (registerGroup != RSHMC804X_VAL_REGISTER_GROUP_QUES_INST_ISUM)
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "STAT:%s:%s?", regGroupArr[registerGroup], regTypeArr[registerType]);
    else
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "STAT:%s%ld:%s?", regGroupArr[registerGroup], ISummary, regTypeArr[registerType]);
    checkErr(rshmc804x_QueryViInt32(instrSession, cmd, registerValue));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Clears all bits in the Questionable Data enable register and the
/// HIFN  Standard Operation enable register.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rshmc804x_init or
/// HIPAR instrSession/rshmc804x_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rshmc804x_StatusPreset(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rshmc804x_SetAttributeViString(instrSession, "", RSHMC804X_ATTR_STATUS_PRESET, NULL));

    checkErr(rshmc804x_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function clears status.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rshmc804x_init or
/// HIPAR instrSession/rshmc804x_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rshmc804x_ClearStatus(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_ClearStatus(instrSession));
    checkErr(rshmc804x_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function returns the ID Query response string.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rshmc804x_init or
/// HIPAR instrSession/rshmc804x_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR IDQueryResponse/Returns the ID Query response string.
ViStatus _VI_FUNC rshmc804x_IDQueryResponse(ViSession instrSession,
                                            ViChar    IDQueryResponse[])
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rshmc804x_QueryViString(instrSession, "*IDN?", 1024, IDQueryResponse));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Stops further command processing until all commands sent before
/// HIFN  *WAI have been executed.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rshmc804x_init or
/// HIPAR instrSession/rshmc804x_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rshmc804x_ProcessAllPreviousCommands(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rshmc804x_WriteInstrData(instrSession, "*WAI"));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function queries the OPC.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rshmc804x_init or
/// HIPAR instrSession/rshmc804x_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR OPC/Queries the OPC.
ViStatus _VI_FUNC rshmc804x_QueryOPC(ViSession instrSession,
                                     ViInt32*  OPC)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rshmc804x_QueryViInt32(instrSession, "*OPC?", OPC));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function switches ON/OFF various error checkings performed inside of the driver.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR optionChecking/This control switches option checking On or Off.
/// HIPAR rangeChecking/This control enables or disables range checking.
/// HIPAR statusChecking/This control enables or disables instrument state checking
ViStatus _VI_FUNC rshmc804x_ConfigureErrorChecking(ViSession instrSession,
                                                   ViBoolean optionChecking,
                                                   ViBoolean rangeChecking,
                                                   ViBoolean statusChecking)
{
    ViStatus         error = VI_SUCCESS;
    RsCoreSessionPtr rsSession = NULL;

    checkErr(RsCore_GetRsSession(instrSession, &rsSession));
    rsSession->optionChecking = optionChecking;
    checkErr(RsCore_SetAttributeViBoolean(instrSession, "", RS_ATTR_QUERY_INSTRUMENT_STATUS, 0, statusChecking));
    checkErr(RsCore_SetAttributeViBoolean(instrSession, "", RS_ATTR_RANGE_CHECK, 0, rangeChecking));

Error:
    return error;
}

/// HIFN This function specifies the minimum timeout value to use (in
/// HIFN milliseconds) when accessing the device associated with the
/// HIFN given session.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rshmc804x_init or
/// HIPAR instrSession/rshmc804x_InitWithOptions function.
/// HIPAR VISATimeout/This control sets the timeout value.
ViStatus _VI_FUNC rshmc804x_SetVISATimeout(ViSession instrSession,
                                           ViUInt32  VISATimeout)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_SetVisaTimeout(instrSession, VISATimeout));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function returns the minimum timeout value to use (in
/// HIFN milliseconds) when accessing the device associated with the
/// HIFN given session.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rshmc804x_init or
/// HIPAR instrSession/rshmc804x_InitWithOptions function.
/// HIPAR VISATimeout/This control gets the timeout value.
ViStatus _VI_FUNC rshmc804x_GetVISATimeout(ViSession instrSession,
                                           ViUInt32* VISATimeout)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetVisaTimeout(instrSession, VISATimeout));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*****************************************************************************
 * Function: Set OPC Timeout
 * Purpose:  Sets new OPC timeout
*****************************************************************************/
ViStatus _VI_FUNC rshmc804x_SetOPCTimeout(ViSession instrSession,
                                          ViInt32   timeout)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rshmc804x_SetAttributeViInt32(instrSession, "", RS_ATTR_OPC_TIMEOUT, timeout),
            3, "Timeout");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*****************************************************************************
 * Function: Get OPC Timeout
 * Purpose:  Gets new OPC timeout
*****************************************************************************/
ViStatus _VI_FUNC rshmc804x_GetOPCTimeout(ViSession instrSession,
                                          ViInt32*  timeout)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rshmc804x_GetAttributeViInt32(instrSession, "", RS_ATTR_OPC_TIMEOUT, timeout),
            3, "Timeout");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*****************************************************************************
 * Function: Get Error and Clear Error Functions
 * Purpose:  These functions enable the instrument driver user to
 *           get or clear the error information the driver associates with the
 *           RS session.
 *****************************************************************************/
ViStatus _VI_FUNC rshmc804x_GetError(ViSession instrSession,
                                     ViStatus* errorCode,
                                     ViInt32   bufferSize,
                                     ViChar    description[])
{
    ViStatus error = VI_SUCCESS;

    // Lock Session - do not jump to the end, even with invalid instrSession the function continues further
    (void)(RsCore_LockSession(instrSession));  

    // Test for nulls and acquire error data
    viCheckParm(RsCore_InvalidNullPointer(instrSession, errorCode), 2, "Null address for Error");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, description), 4, "Null address for Description");

    checkErr(RsCore_GetErrorCompleteDescription(instrSession, &rshmc804x_utility_error_message, errorCode, bufferSize, description)
    );

Error:
    // Unlock Session
    // IGNORE: ERROR!!! Missing Lock
    return error;
}

ViStatus _VI_FUNC rshmc804x_ClearError(ViSession instrSession)
{
    return RsCore_ClearErrorInfo(instrSession);
}

/*****************************************************************************
 * Function: WriteInstrData and ReadInstrData Functions
 * Purpose:  These functions enable the instrument driver user to
 *           write and read commands directly to and from the instrument.
 *
 *           Note:  These functions bypass the RS attribute state caching.
 *                  WriteInstrData invalidates the cached values for all
 *                  attributes.
 *****************************************************************************/

ViStatus _VI_FUNC rshmc804x_WriteInstrData(ViSession instrSession,
                                           ViString  writeBuffer)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    error = RsCore_Write(instrSession, writeBuffer);

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*****************************************************************************
 * Function: Query ViBoolean
 * Purpose:  This function queries the ViBoolean value.
 *****************************************************************************/
ViStatus _VI_FUNC rshmc804x_QueryViBoolean(ViSession  instrSession,
                                           ViString   cmd,
                                           ViBoolean* value)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidStringLength(instrSession, cmd, 1, -1),
        2, "Cmd (null string length)");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, value),
        3, "Value");

    checkErr(RsCore_QueryViBoolean(instrSession, cmd, value));

    checkErr(rshmc804x_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*****************************************************************************
 * Function: Query ViInt32
 * Purpose:  This function queries the ViInt32 value.
 *****************************************************************************/
ViStatus _VI_FUNC rshmc804x_QueryViInt32(ViSession instrSession,
                                         ViString  cmd,
                                         ViInt32*  value)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidStringLength(instrSession, cmd, 1, -1),
        2, "Cmd (null string length)");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, value), 3, "Value");

    checkErr(RsCore_QueryViInt32(instrSession, cmd, value));

    checkErr(rshmc804x_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*****************************************************************************
 * Function: Query ViReal64
 * Purpose:  This function queries the ViReal64 value.
 *****************************************************************************/
ViStatus _VI_FUNC rshmc804x_QueryViReal64(ViSession instrSession,
                                          ViString  cmd,
                                          ViReal64* value)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidStringLength(instrSession, cmd, 1, -1),
        2, "Cmd (null string length)");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, value), 3, "Value");

    checkErr(RsCore_QueryViReal64(instrSession, cmd, value));

    checkErr(rshmc804x_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/*****************************************************************************
 * Function: Query ViString
 * Purpose:  This function queries the ViString value.
 *****************************************************************************/
ViStatus _VI_FUNC rshmc804x_QueryViString(ViSession instrSession,
                                          ViString  cmd,
                                          ViInt32   bufferSize,
                                          ViChar    value[])
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidStringLength(instrSession, cmd, 1, -1),
        2, "Cmd (null string length)");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, value), 3, "Value");

    checkErr(RsCore_QueryViStringUnknownLengthToUserBuffer(instrSession, cmd, bufferSize, value, NULL));

    checkErr(rshmc804x_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  The DATA? query responses screenshot data in binary format.
/// HIFN  control will be activated.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rshmc804x_init or
/// HIPAR instrSession/rshmc804x_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR destination/Defines destination file to which the data transfer from instrument to
/// HIPAR destination/control computer applies.
ViStatus _VI_FUNC rshmc804x_ImageSnapshot(ViSession instrSession,
                                          ViString  destination)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidNullPointer(instrSession, destination), 2, "Destination");

    checkErr(RsCore_QueryBinaryDataBlockToFile(instrSession, "HCOP:DATA?", destination, RS_VAL_TRUNCATE));

    checkErr(rshmc804x_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the data format of the screenshot.
/// HIFN  control will be activated.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rshmc804x_init or
/// HIPAR instrSession/rshmc804x_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR imageFormat/Selects the data format of the screenshot.
ViStatus _VI_FUNC rshmc804x_ConfigureImageFormat(ViSession instrSession,
                                                 ViInt32   imageFormat)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rshmc804x_SetAttributeViInt32(instrSession, "", RSHMC804X_ATTR_HCOPY_IMAGE_FORMAT, imageFormat),
            2, "Image Format");

    checkErr(rshmc804x_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Returns the width and height of the screenshot.
/// HIFN  control will be activated.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rshmc804x_init or
/// HIPAR instrSession/rshmc804x_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR imageWidth/Returns the width of the screenshot.
/// HIPAR imageHeight/Returns the height of the screenshot.
ViStatus _VI_FUNC rshmc804x_QueryImageSize(ViSession instrSession,
                                           ViInt32*  imageWidth,
                                           ViInt32*  imageHeight)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rshmc804x_GetAttributeViInt32(instrSession, "", RSHMC804X_ATTR_HCOPY_IMAGE_WIDTH, imageWidth),
            2, "Image Width");

    viCheckParm(rshmc804x_GetAttributeViInt32(instrSession, "", RSHMC804X_ATTR_HCOPY_IMAGE_HEIGHT, imageHeight),
            3, "Image Height");

    checkErr(rshmc804x_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*****************************************************************************
 * Function: rshmc804x_close
 * Purpose:  This function closes the instrument.
 *
 *           Note:  This function must unlock the session before calling
 *           Rs_Dispose.
 *****************************************************************************/
ViStatus _VI_FUNC rshmc804x_close(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_ViClose(instrSession));

Error:
    (void)RsCore_Dispose(instrSession);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Combined setting of voltage and current for each channel.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rshmc804x_init or
/// HIPAR instrSession/rshmc804x_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Numerical selection of a channel.
/// HIPAR voltage/Defines the voltage value of the selected channel.
/// HIPAR current/Defines the current value of the selected channel.
ViStatus _VI_FUNC rshmc804x_ConfigureOutput(ViSession instrSession,
                                            ViInt32   channel,
                                            ViReal64  voltage,
                                            ViReal64  current)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, channel, 1, 3),
            2, "Channel");

    if (rshmc804x_IsInstrumentNgxFamily(instrSession))
    {
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "INST OUT%ld;APPL \"%f,%f\"", channel, voltage, current);
    }
    else
    {
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "APPL %f,%f,OUT%ld", voltage, current, channel);
    }

    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rshmc804x_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the incremental step size
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rshmc804x_init or
/// HIPAR instrSession/rshmc804x_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Numerical selection of a channel.
/// HIPAR stepSizeType/The kind of step size to be configured.
/// HIPAR stepSize/Defines the selected step size (voltage level/current limit).
ViStatus _VI_FUNC rshmc804x_ConfigureStepSize(ViSession instrSession,
                                              ViInt32   channel,
                                              ViInt32   stepSizeType,
                                              ViReal64  stepSize)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rshmc804x_SetAttributeViInt32(instrSession, "", RSHMC804X_ATTR_OUTPUTS_SELECTED_CHANNEL, channel),
            2, "Channel");

    switch (stepSizeType)
    {
    case RSHMC804X_VAL_STEP_SIZE_VOLTAGE:
        viCheckParm(rshmc804x_SetAttributeViReal64(instrSession, "", RSHMC804X_ATTR_OUTPUTS_VOLTAGE_STEP_SIZE, stepSize),
                4, "Step Size");
        break;
    case RSHMC804X_VAL_STEP_SIZE_CURRENT:
        viCheckParm(rshmc804x_SetAttributeViReal64(instrSession, "", RSHMC804X_ATTR_OUTPUTS_CURRENT_STEP_SIZE, stepSize),
                4, "Step Size");
        break;
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the voltage value of the selected channel.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rshmc804x_init or
/// HIPAR instrSession/rshmc804x_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Numerical selection of a channel.
/// HIPAR level/Defines the voltage level.
ViStatus _VI_FUNC rshmc804x_ConfigureVoltageLevel(ViSession instrSession,
                                                  ViInt32   channel,
                                                  ViReal64  level)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rshmc804x_SetAttributeViInt32(instrSession, "", RSHMC804X_ATTR_OUTPUTS_SELECTED_CHANNEL, channel),
            2, "Channel");

    viCheckParm(rshmc804x_SetAttributeViReal64(instrSession, "", RSHMC804X_ATTR_OUTPUTS_VOLTAGE_LEVEL, level),
            3, "Level");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Enables and configures the OVP (over voltage protection) function.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rshmc804x_init or
/// HIPAR instrSession/rshmc804x_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Numerical selection of a channel.
/// HIPAR enabled/Enables (ON/1) / Disables (OFF/0) the OVP (over voltage protection) function.
/// HIPAR limit/Sets the voltage level of the OVP function when the OVP will be tripped.
ViStatus _VI_FUNC rshmc804x_ConfigureOVP(ViSession instrSession,
                                         ViInt32   channel,
                                         ViBoolean enabled,
                                         ViReal64  limit)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rshmc804x_SetAttributeViInt32(instrSession, "", RSHMC804X_ATTR_OUTPUTS_SELECTED_CHANNEL, channel),
            2, "Channel");

    viCheckParm(rshmc804x_SetAttributeViReal64(instrSession, "", RSHMC804X_ATTR_OUTPUTS_OVP_LIMIT, limit),
            4, "Limit");

    viCheckParm(rshmc804x_SetAttributeViBoolean(instrSession, "", RSHMC804X_ATTR_OUTPUTS_OVP_ENABLED, enabled),
            3, "Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the OVP function mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rshmc804x_init or
/// HIPAR instrSession/rshmc804x_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Numerical selection of a channel.
/// HIPAR OVPMode/Selects the OVP function mode.
ViStatus _VI_FUNC rshmc804x_ConfigureOVPMode(ViSession instrSession,
                                             ViInt32   channel,
                                             ViInt32   OVPMode)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rshmc804x_SetAttributeViInt32(instrSession, "", RSHMC804X_ATTR_OUTPUTS_SELECTED_CHANNEL, channel),
            2, "Channel");

    viCheckParm(rshmc804x_SetAttributeViInt32(instrSession, "", RSHMC804X_ATTR_OUTPUTS_OVP_MODE, OVPMode),
            3, "OVP Mode");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Enables and configures the OPP (over power protection) function.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rshmc804x_init or
/// HIPAR instrSession/rshmc804x_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Numerical selection of a channel.
/// HIPAR enabled/Enables (ON/1) / Disables (OFF/0) the OPP (over power protection) function.
/// HIPAR limit/Sets the power level of the OPP function when the OPP will be tripped.
ViStatus _VI_FUNC rshmc804x_ConfigureOPP(ViSession instrSession,
                                         ViInt32   channel,
                                         ViBoolean enabled,
                                         ViReal64  limit)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rshmc804x_SetAttributeViInt32(instrSession, "", RSHMC804X_ATTR_OUTPUTS_SELECTED_CHANNEL, channel),
            2, "Channel");

    viCheckParm(rshmc804x_SetAttributeViReal64(instrSession, "", RSHMC804X_ATTR_OUTPUTS_OPP_LIMIT, limit),
            4, "Limit");

    viCheckParm(rshmc804x_SetAttributeViBoolean(instrSession, "", RSHMC804X_ATTR_OUTPUTS_OPP_ENABLED, enabled),
            3, "Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Enables and configures the EasyRamp function.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rshmc804x_init or
/// HIPAR instrSession/rshmc804x_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Numerical selection of a channel.
/// HIPAR enabled/Enables (ON/1) / Disables (OFF/0) the EasyRamp function.
/// HIPAR duration/Sets the duration of the voltage ramp.
ViStatus _VI_FUNC rshmc804x_ConfigureRamp(ViSession instrSession,
                                          ViInt32   channel,
                                          ViBoolean enabled,
                                          ViReal64  duration)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rshmc804x_SetAttributeViInt32(instrSession, "", RSHMC804X_ATTR_OUTPUTS_SELECTED_CHANNEL, channel),
            2, "Channel");

    viCheckParm(rshmc804x_SetAttributeViReal64(instrSession, "", RSHMC804X_ATTR_OUTPUTS_RAMP_DURATION, duration),
            4, "Duration");

    viCheckParm(rshmc804x_SetAttributeViBoolean(instrSession, "", RSHMC804X_ATTR_OUTPUTS_RAMP_ENABLED, enabled),
            3, "Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Enables and configures the EasyRamp function.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rshmc804x_init or
/// HIPAR instrSession/rshmc804x_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Numerical selection of a channel.
/// HIPAR enabled/Enables (ON/1) / Disables (OFF/0) the analog input on the rear connector.
/// HIPAR input/Selects the input unit of the analog input connector.
/// HIPAR mode/Selects the mode of the analog input connector.
/// HIPAR threshold/Sets the threshold for the STEP input mode.
ViStatus _VI_FUNC rshmc804x_ConfigureAnalogInput(ViSession instrSession,
                                                 ViInt32   channel,
                                                 ViBoolean enabled,
                                                 ViInt32   input,
                                                 ViInt32   mode,
                                                 ViReal64  threshold)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rshmc804x_SetAttributeViInt32(instrSession, "", RSHMC804X_ATTR_OUTPUTS_SELECTED_CHANNEL, channel),
            2, "Channel");

    viCheckParm(rshmc804x_SetAttributeViInt32(instrSession, "", RSHMC804X_ATTR_OUTPUTS_AINPUT_INPUT, input),
            4, "Input");

    viCheckParm(rshmc804x_SetAttributeViInt32(instrSession, "", RSHMC804X_ATTR_OUTPUTS_AINPUT_MODE, mode),
            5, "Mode");

    if (mode == RSHMC804X_VAL_AINPUT_MODE_STEP)
    {
        viCheckParm(rshmc804x_SetAttributeViReal64(instrSession, "", RSHMC804X_ATTR_OUTPUTS_AINPUT_THRESHOLD, threshold),
                6, "Threshold");
    }

    viCheckParm(rshmc804x_SetAttributeViBoolean(instrSession, "", RSHMC804X_ATTR_OUTPUTS_AINPUT_ENABLED, enabled),
            3, "Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the voltage value of the selected channel.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rshmc804x_init or
/// HIPAR instrSession/rshmc804x_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Numerical selection of a channel.
/// HIPAR limit/Defines the current value of the selected channel.
ViStatus _VI_FUNC rshmc804x_ConfigureCurrentLimit(ViSession instrSession,
                                                  ViInt32   channel,
                                                  ViReal64  limit)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rshmc804x_SetAttributeViInt32(instrSession, "", RSHMC804X_ATTR_OUTPUTS_SELECTED_CHANNEL, channel),
            2, "Channel");

    viCheckParm(rshmc804x_SetAttributeViReal64(instrSession, "", RSHMC804X_ATTR_OUTPUTS_CURRENT_LIMIT, limit),
            3, "Limit");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the power supply�s output range on an output. One parameter specifies
/// HIFN  whether to configure the voltage or current range, and the other parameter is the
/// HIFN  value to which to set the range.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rshmc804x_init or
/// HIPAR instrSession/rshmc804x_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Numerical selection of a channel.
/// HIPAR rangeType/The kind of range to be configured.
/// HIPAR range/Defines the selected range (voltage level/current limit).
ViStatus _VI_FUNC rshmc804x_ConfigureOutputRange(ViSession instrSession,
                                                 ViInt32   channel,
                                                 ViInt32   rangeType,
                                                 ViReal64  range)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rshmc804x_SetAttributeViInt32(instrSession, "", RSHMC804X_ATTR_OUTPUTS_SELECTED_CHANNEL, channel),
            2, "Channel");

    switch (rangeType)
    {
    case RSHMC804X_VAL_RANGE_VOLTAGE:
        viCheckParm(rshmc804x_SetAttributeViReal64(instrSession, "", RSHMC804X_ATTR_OUTPUTS_VOLTAGE_LEVEL, range),
                4, "Range");
        break;
    case RSHMC804X_VAL_RANGE_CURRENT:
        viCheckParm(rshmc804x_SetAttributeViReal64(instrSession, "", RSHMC804X_ATTR_OUTPUTS_CURRENT_LIMIT, range),
                4, "Range");
        break;
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Turns the output on or off.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rshmc804x_init or
/// HIPAR instrSession/rshmc804x_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Numerical selection of a channel.
/// HIPAR enabled/Turns the output on or off.
ViStatus _VI_FUNC rshmc804x_ConfigureOutputEnabled(ViSession instrSession,
                                                   ViInt32   channel,
                                                   ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rshmc804x_SetAttributeViInt32(instrSession, "", RSHMC804X_ATTR_OUTPUTS_SELECTED_CHANNEL, channel),
            2, "Channel");

    viCheckParm(rshmc804x_SetAttributeViBoolean(instrSession, "", RSHMC804X_ATTR_OUTPUTS_ENABLED, enabled),
            3, "Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Turns the output ON or OFF. Compared to
/// HIFN  rshmc804x_ConfigureOutputEnabled, it doesn't change the Master
/// HIFN  switch settings.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rshmc804x_init or
/// HIPAR instrSession/rshmc804x_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Numerical selection of a channel.
/// HIPAR enabled/Turns the output ON or OFF.
ViStatus _VI_FUNC rshmc804x_ConfigureChannelOnlyEnabled(ViSession instrSession,
                                                        ViInt32   channel,
                                                        ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rshmc804x_SetAttributeViInt32(instrSession, "", RSHMC804X_ATTR_OUTPUTS_SELECTED_CHANNEL, channel),
            2, "Channel");

    viCheckParm(rshmc804x_SetAttributeViBoolean(instrSession, "", RSHMC804X_ATTR_CHANNEL_ONLY_ENABLED, enabled),
            3, "Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Switchs ON/OFF all previous selected channels simultaneously.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rshmc804x_init or
/// HIPAR instrSession/rshmc804x_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR enabled/Turns the all output on or off.
ViStatus _VI_FUNC rshmc804x_ConfigureMasterOutputEnabled(ViSession instrSession,
                                                         ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rshmc804x_SetAttributeViBoolean(instrSession, "", RSHMC804X_ATTR_OUTPUTS_MASTER_ENABLED, enabled),
            2, "Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function returns the maximum programmable current limit that
/// HIFN  the power supply accepts for a particular voltage level on an output.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rshmc804x_init or
/// HIPAR instrSession/rshmc804x_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Numerical selection of a channel.
/// HIPAR voltageLevel/The voltage level for which to determine the maximum programmable current limit.
/// HIPAR maxCurrentLimit/Returns the maximum programmable current limit for the specified voltage level.
ViStatus _VI_FUNC rshmc804x_QueryMaxCurrentLimit(ViSession instrSession,
                                                 ViInt32   channel,
                                                 ViReal64  voltageLevel,
                                                 ViReal64* maxCurrentLimit)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rshmc804x_SetAttributeViInt32(instrSession, "", RSHMC804X_ATTR_OUTPUTS_SELECTED_CHANNEL, channel),
            2, "Channel");

    viCheckParm(rshmc804x_SetAttributeViReal64(instrSession, "", RSHMC804X_ATTR_OUTPUTS_VOLTAGE_LEVEL, voltageLevel),
            3, "Voltage Level");

    checkErr(rshmc804x_QueryViReal64(instrSession, "CURR? MAX", maxCurrentLimit));

    checkErr(rshmc804x_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function returns the maximum programmable voltage level that the power supply
/// HIFN  accepts for a particular current limit on an output.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rshmc804x_init or
/// HIPAR instrSession/rshmc804x_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Numerical selection of a channel.
/// HIPAR currentLimit/The current limit for which to determine the maximum programmable voltage level.
/// HIPAR maxVoltageLevel/Returns the maximum programmable voltage level for the specified current limit.
ViStatus _VI_FUNC rshmc804x_QueryMaxVoltageLevel(ViSession instrSession,
                                                 ViInt32   channel,
                                                 ViReal64  currentLimit,
                                                 ViReal64* maxVoltageLevel)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rshmc804x_SetAttributeViInt32(instrSession, "", RSHMC804X_ATTR_OUTPUTS_SELECTED_CHANNEL, channel),
            2, "Channel");

    viCheckParm(rshmc804x_SetAttributeViReal64(instrSession, "", RSHMC804X_ATTR_OUTPUTS_CURRENT_LIMIT, currentLimit),
            3, "Current Limit");

    checkErr(rshmc804x_QueryViReal64(instrSession, "VOLT? MAX", maxVoltageLevel));
    checkErr(rshmc804x_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the output fusion state for 2-channel model.
/// HIFN
/// HIFN     Note(s):
/// HIFN
/// HIFN     (1) This function is available only for NGA instrument
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rshmc804x_init or
/// HIPAR instrSession/rshmc804x_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Numerical selection of a channel
/// HIPAR channelFusion/Sets the output fusion state for 2-channel model.
ViStatus _VI_FUNC rshmc804x_ConfigureChannelFusion(ViSession instrSession,
                                                   ViInt32   channel,
                                                   ViInt32   channelFusion)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rshmc804x_SetAttributeViInt32(instrSession, "", RSHMC804X_ATTR_OUTPUTS_SELECTED_CHANNEL, channel),
            2, "Channel");

    viCheckParm(rshmc804x_SetAttributeViInt32(instrSession, "", RSHMC804X_ATTR_CHANNEL_FUSION, channelFusion),
            3, "Channel Fusion");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the output operating mode of the specified channel.
/// HIFN
/// HIFN     Note(s):
/// HIFN
/// HIFN     (1) This function is available only for NGF instrument
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rshmc804x_init or
/// HIPAR instrSession/rshmc804x_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Numerical selection of a channel
/// HIPAR mode/Sets the output operating mode of the specified channel.
ViStatus _VI_FUNC rshmc804x_ConfigureOutputMode(ViSession instrSession,
                                                ViInt32   channel,
                                                ViInt32   mode)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rshmc804x_SetAttributeViInt32(instrSession, "", RSHMC804X_ATTR_OUTPUTS_SELECTED_CHANNEL, channel),
            2, "Channel");

    viCheckParm(rshmc804x_SetAttributeViInt32(instrSession, "", RSHMC804X_ATTR_MODE, mode),
            3, "Mode");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the output operating mode of the previous selected channel.
/// HIFN
/// HIFN     Note(s):
/// HIFN
/// HIFN     (1) This function is available only for NGA or NGF instrument
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rshmc804x_init or
/// HIPAR instrSession/rshmc804x_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Numerical selection of a channel
/// HIPAR mode/Queries the output operating mode of the previous selected channel.
ViStatus _VI_FUNC rshmc804x_QueryOutputMode(ViSession instrSession,
                                            ViInt32   channel,
                                            ViInt32*  mode)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rshmc804x_SetAttributeViInt32(instrSession, "", RSHMC804X_ATTR_OUTPUTS_SELECTED_CHANNEL, channel),
            2, "Channel");

    viCheckParm(rshmc804x_GetAttributeViInt32(instrSession, "", RSHMC804X_ATTR_QUERY_MODE, mode),
            3, "Mode");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the DVM mode state for the previous selected channel
/// HIFN
/// HIFN     Note(s):
/// HIFN
/// HIFN     (1) This function is available only for NGF instrument
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rshmc804x_init or
/// HIPAR instrSession/rshmc804x_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Numerical selection of a channel
/// HIPAR enabled/Configures the DVM mode state for the previous selected channel
ViStatus _VI_FUNC rshmc804x_ConfigureVoltmeterEnabled(ViSession instrSession,
                                                      ViInt32   channel,
                                                      ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rshmc804x_SetAttributeViInt32(instrSession, "", RSHMC804X_ATTR_OUTPUTS_SELECTED_CHANNEL, channel),
            2, "Channel");

    viCheckParm(rshmc804x_SetAttributeViBoolean(instrSession, "", RSHMC804X_ATTR_VOLTMETER_ENABLED, enabled),
            3, "Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Returns the measured voltmeter value of the previous selected channel
/// HIFN
/// HIFN     Note(s):
/// HIFN
/// HIFN     (1) This function is available only for NGF instrument
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rshmc804x_init or
/// HIPAR instrSession/rshmc804x_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Numerical selection of a channel
/// HIPAR voltmeterValue/Returns the measured voltmeter value of the previous selected channel
ViStatus _VI_FUNC rshmc804x_QueryVoltmeterValue(ViSession instrSession,
                                                ViInt32   channel,
                                                ViReal64* voltmeterValue)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rshmc804x_SetAttributeViInt32(instrSession, "", RSHMC804X_ATTR_OUTPUTS_SELECTED_CHANNEL, channel),
            2, "Channel");

    viCheckParm(rshmc804x_GetAttributeViReal64(instrSession, "", RSHMC804X_ATTR_QUERY_VOLTMETER_VALUE, voltmeterValue),
            3, "Voltmeter Value");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Displays a text message box on the front display.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rshmc804x_init or
/// HIPAR instrSession/rshmc804x_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR text/The text to display in the message box.
ViStatus _VI_FUNC rshmc804x_ShowMessageBox(ViSession instrSession,
                                           ViString  text)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rshmc804x_SetAttributeViString(instrSession, "", RSHMC804X_ATTR_DISPLAY_TEXT, text),
            2, "Text");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Clears the text message box on the front display.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rshmc804x_init or
/// HIPAR instrSession/rshmc804x_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rshmc804x_ClearMessageBox(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rshmc804x_SetAttributeViString(instrSession, "", RSHMC804X_ATTR_DISPLAY_CLEAR, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Enables (On/1) / Disables (Off/0) the fuse for the selected channel and configures delay.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rshmc804x_init or
/// HIPAR instrSession/rshmc804x_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Numerical selection of a channel
/// HIPAR enabled/Enables (On/1) / Disables (Off/0) the fuse for the selected channel.
/// HIPAR delay/Sets the fuse delay.
ViStatus _VI_FUNC rshmc804x_ConfigureFuse(ViSession instrSession,
                                          ViInt32   channel,
                                          ViBoolean enabled,
                                          ViReal64  delay)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rshmc804x_SetAttributeViInt32(instrSession, "", RSHMC804X_ATTR_OUTPUTS_SELECTED_CHANNEL, channel),
            2, "Channel");

    viCheckParm(rshmc804x_SetAttributeViReal64(instrSession, "", RSHMC804X_ATTR_OUTPUTS_FUSE_DELAY, delay),
            4, "Delay");

    viCheckParm(rshmc804x_SetAttributeViBoolean(instrSession, "", RSHMC804X_ATTR_OUTPUTS_FUSE_ENABLED, enabled),
            3, "Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Combines or unlinking the channel fuses (Fuse Linking or unlinking).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rshmc804x_init or
/// HIPAR instrSession/rshmc804x_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Numerical selection of a channel
/// HIPAR fuseLinkingType/The kind of linking to be configured.
/// HIPAR channelFuses/Numerical selection of a channel fuses.
ViStatus _VI_FUNC rshmc804x_ConfigureFuseLinking(ViSession instrSession,
                                                 ViInt32   channel,
                                                 ViInt32   fuseLinkingType,
                                                 ViInt32   channelFuses)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rshmc804x_SetAttributeViInt32(instrSession, "", RSHMC804X_ATTR_OUTPUTS_SELECTED_CHANNEL, channel),
            2, "Channel");

    switch (fuseLinkingType)
    {
    case RSHMC804X_VAL_FUSE_LINKING:
        viCheckParm(rshmc804x_SetAttributeViInt32(instrSession, "", RSHMC804X_ATTR_OUTPUTS_FUSE_LINK, channelFuses),
                4, "Channel Fuses");
        break;
    case RSHMC804X_VAL_FUSE_UNLINKING:
        viCheckParm(rshmc804x_SetAttributeViInt32(instrSession, "", RSHMC804X_ATTR_OUTPUTS_FUSE_UNLINK, channelFuses),
                4, "Channel Fuses");
        break;
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the arbitrary function of the selected channel.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rshmc804x_init or
/// HIPAR instrSession/rshmc804x_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Numerical selection of a channel
/// HIPAR enabled/Enables (On/1) / Disables (Off/0) the arbitrary function of the selected channel.
/// HIPAR repetitions/Sets the arbitrary waveform repetitions.
/// HIPAR endpoint/Sets the endpoint of the arbitrary waveform.
ViStatus _VI_FUNC rshmc804x_ConfigureArbitrary(ViSession instrSession,
                                               ViInt32   channel,
                                               ViBoolean enabled,
                                               ViInt32   repetitions,
                                               ViInt32   endpoint)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rshmc804x_SetAttributeViInt32(instrSession, "", RSHMC804X_ATTR_OUTPUTS_SELECTED_CHANNEL, channel),
            2, "Channel");

    viCheckParm(rshmc804x_SetAttributeViInt32(instrSession, "", RSHMC804X_ATTR_OUTPUTS_ARBITRARY_REPETITIONS, repetitions),
            4, "Repetitions");

    viCheckParm(rshmc804x_SetAttributeViInt32(instrSession, "", RSHMC804X_ATTR_OUTPUTS_ARBITRARY_ENDPOINT, endpoint),
            4, "Endpoint");

    viCheckParm(rshmc804x_SetAttributeViBoolean(instrSession, "", RSHMC804X_ATTR_OUTPUTS_ARBITRARY_ENABLED, enabled),
            3, "Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Set the filename, device and operation for arbitrary.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rshmc804x_init or
/// HIPAR instrSession/rshmc804x_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Numerical selection of a channel
/// HIPAR filename/Selects the filename.
/// HIPAR device/Selects the device.
/// HIPAR operation/Selects the file operation.
ViStatus _VI_FUNC rshmc804x_ConfigureArbitraryFile(ViSession instrSession,
                                                   ViInt32   channel,
                                                   ViString  filename,
                                                   ViInt32   device,
                                                   ViInt32   operation)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rshmc804x_SetAttributeViInt32(instrSession, "", RSHMC804X_ATTR_OUTPUTS_SELECTED_CHANNEL, channel),
            2, "Channel");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "ARB:FNAM '%s',%s", filename, deviceArr[device]);
    checkErr(RsCore_Write(instrSession, cmd));

    switch (operation)
    {
    case RSHMC804X_VAL_FILE_OPER_SAVE:
        checkErr(rshmc804x_SetAttributeViString(instrSession, "", RSHMC804X_ATTR_OUTPUTS_ARBITRARY_SAVE_FILE, NULL));
        break;
    case RSHMC804X_VAL_FILE_OPER_LOAD:
        checkErr(rshmc804x_SetAttributeViString(instrSession, "", RSHMC804X_ATTR_OUTPUTS_ARBITRARY_LOAD_FILE, NULL));
        break;
    }

    checkErr(rshmc804x_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the trigger mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rshmc804x_init or
/// HIPAR instrSession/rshmc804x_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Numerical selection of a channel
/// HIPAR triggerMode/Selects the trigger mode.
ViStatus _VI_FUNC rshmc804x_ConfigureArbitraryTriggerMode(ViSession instrSession,
                                                          ViInt32   channel,
                                                          ViInt32   triggerMode)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rshmc804x_SetAttributeViInt32(instrSession, "", RSHMC804X_ATTR_OUTPUTS_SELECTED_CHANNEL, channel),
            2, "Channel");

    viCheckParm(rshmc804x_SetAttributeViInt32(instrSession, "", RSHMC804X_ATTR_OUTPUTS_ARBITRARY_TRIGGER_MODE, triggerMode),
            3, "Trigger Mode");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the arbitrary waveform values for the selected channel.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rshmc804x_init or
/// HIPAR instrSession/rshmc804x_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Numerical selection of a channel
/// HIPAR numberOfValues/The number of values written.
/// HIPAR voltage/Writes an array of voltage.
/// HIPAR current/Writes an array of current.
/// HIPAR time/Writes an array of time.
/// HIPAR interpolationMode/Writes an array of interpolation mode.
ViStatus _VI_FUNC rshmc804x_WriteArbitraryData(ViSession instrSession,
                                               ViInt32   channel,
                                               ViInt32   numberOfValues,
                                               ViReal64  voltage[],
                                               ViReal64  current[],
                                               ViReal64  time[],
                                               ViInt32   interpolationMode[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViChar*  pbuf = cmd;
    ViInt32  i;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rshmc804x_SetAttributeViInt32(instrSession, "", RSHMC804X_ATTR_OUTPUTS_SELECTED_CHANNEL, channel),
            2, "Channel");

    if (numberOfValues > 0)
    {
        pbuf += sprintf(pbuf, "ARB:DATA ");

        for (i = 0; i < numberOfValues; i++)
            pbuf += sprintf(pbuf, "%.12lf,%.12lf,%.12lf,%ld,", voltage[i], current[i], time[i], interpolationMode[i]);

        *--pbuf = '\0';
        checkErr(RsCore_Write(instrSession, cmd));
    }
    checkErr(rshmc804x_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the arbitrary waveform values of the selected channel.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rshmc804x_init or
/// HIPAR instrSession/rshmc804x_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Numerical selection of a channel
/// HIPAR timeOut_msec/Sets the maximum amount of time (in milli seconds) that the
/// HIPAR timeOut_msec/controller will wait for the instrument to respond before it
/// HIPAR timeOut_msec/times out.
/// HIPAR arraySize/This parameter determines the size of arrays.
/// HIPAR voltage/Returns an array of voltage.
/// HIPAR current/Returns an array of current.
/// HIPAR time/Returns an array of time.
/// HIPAR interpolationMode/Returns an array of interpolation mode.
/// HIPAR returnedValues/Returns the number of returned values.
ViStatus _VI_FUNC rshmc804x_ReadArbitraryData(ViSession instrSession,
                                              ViInt32   channel,
                                              ViInt32   timeOut_msec,
                                              ViInt32   arraySize,
                                              ViReal64  voltage[],
                                              ViReal64  current[],
                                              ViReal64  time[],
                                              ViInt32   interpolationMode[],
                                              ViInt32*  returnedValues)
{
    ViStatus      error = VI_SUCCESS;
    ViChar*       buffer = NULL;
    ViChar*       pbuff;
    ViInt32       i;
    ViInt32       cnt = 1;
    ViInt32       byteCount;
    timeOut_msec; // unused

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rshmc804x_SetAttributeViInt32(instrSession, "", RSHMC804X_ATTR_OUTPUTS_SELECTED_CHANNEL, channel),
            2, "Channel");

    checkErr(RsCore_QueryViStringUnknownLength(instrSession, "ARB:DATA?", &buffer));
    byteCount = (ViInt32)strlen(buffer);

    if (buffer[byteCount - 2] == ',')
        buffer[byteCount - 2] = 0;

    pbuff = strtok(buffer, ",");
    i = 0;
    do
    {
        voltage[i] = RsCore_Convert_String_To_ViReal64(pbuff);
        pbuff = strtok(NULL, ",");
        current[i] = RsCore_Convert_String_To_ViReal64(pbuff);
        pbuff = strtok(NULL, ",");
        time[i] = RsCore_Convert_String_To_ViReal64(pbuff);
        pbuff = strtok(NULL, ",");
        interpolationMode[i] = RsCore_Convert_String_To_ViInt32(pbuff);
        pbuff = strtok(NULL, ",");
        i++;
    }
    while ((pbuff != NULL) && (i < arraySize));

    if (returnedValues != NULL)
    {
        while ((pbuff != NULL))
        {
            pbuff = strtok(NULL, ",");
            cnt++;
        }

        *returnedValues = i + (cnt + 1) / 4;
    }

    checkErr(rshmc804x_CheckStatus(instrSession));

Error:
    if (buffer)
        free(buffer);

    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Set the filename, format and device for logging.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rshmc804x_init or
/// HIPAR instrSession/rshmc804x_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR filename/Selects the filename.
/// HIPAR device/Selects the device.
/// HIPAR format/Selects the format for logging.
ViStatus _VI_FUNC rshmc804x_ConfigureLoggingFile(ViSession instrSession,
                                                 ViString  filename,
                                                 ViInt32   device,
                                                 ViInt32   format)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidNullPointer(instrSession, filename), 2, "Filename");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "LOG:FNAM '%s',%s", filename, deviceArr[device]);
    checkErr(RsCore_Write(instrSession, cmd));

    viCheckParm(rshmc804x_SetAttributeViInt32(instrSession, "", RSHMC804X_ATTR_OUTPUTS_MEAS_LOG_FORMAT, format),
            4, "Format");

    checkErr(rshmc804x_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Set the filename, format and device for logging.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rshmc804x_init or
/// HIPAR instrSession/rshmc804x_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR filename/Selects the filename.
/// HIPAR device/Selects the device.
/// HIPAR bufferSize/Pass the number of bytes in the ViChar array you specify for the
/// HIPAR bufferSize/"File Content" control.
/// HIPAR fileContent/Read the file "Filename" and returns the file content.
ViStatus _VI_FUNC rshmc804x_ReadFileFromInstrument(ViSession instrSession,
                                                   ViString  filename,
                                                   ViInt32   device,
                                                   ViInt32   bufferSize,
                                                   ViChar    fileContent[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViInt64  dataSize;
    ViByte*  data = NULL;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidNullPointer(instrSession, filename), 2, "Filename");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "DATA:DATA? '%s',%s", filename, deviceArr[device]);
    checkErr(RsCore_QueryBinaryDataBlock(instrSession, cmd, VI_TRUE, NULL, &data, &dataSize));
    checkErr(RsCore_CopyToUserBufferBinData((ViBuf)fileContent, bufferSize, data, dataSize));

Error:
    if (data)
        free(data);

    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Delete the file "Filename" (INT -> Internal Memory, EXT -> USB Stick, DEF -> Internal Memory).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rshmc804x_init or
/// HIPAR instrSession/rshmc804x_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR filename/Selects the filename.
/// HIPAR device/Selects the device.
ViStatus _VI_FUNC rshmc804x_Delete(ViSession instrSession,
                                   ViString  filename,
                                   ViInt32   device)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidNullPointer(instrSession, filename), 2, "Filename");
    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "DATA:DEL '%s',%s", filename, deviceArr[device]);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rshmc804x_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Read the number of measurements from "Filename" (INT -> Internal Memory, EXT -> USB Stick, DEF -> Internal Memory).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rshmc804x_init or
/// HIPAR instrSession/rshmc804x_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR filename/Selects the filename.
/// HIPAR device/Selects the device.
/// HIPAR points/Returns the number of measurements.
ViStatus _VI_FUNC rshmc804x_ReadDataPoints(ViSession instrSession,
                                           ViString  filename,
                                           ViInt32   device,
                                           ViInt32*  points)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidNullPointer(instrSession, filename),
        2, "Filename");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "DATA:POIN? '%s',%s", filename, deviceArr[device]);
    checkErr(rshmc804x_QueryViInt32(instrSession, cmd, points));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  List all available files of the selected memory (INT -> Internal Memory, EXT -> USB Stick, DEF -> Internal Memory).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rshmc804x_init or
/// HIPAR instrSession/rshmc804x_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR device/Selects the device.
/// HIPAR bufferSize/Pass the number of bytes in the ViChar array you specify for the Output parameter.
/// HIPAR catalog/Returns the files in the indicated directory as comma separated strings.
ViStatus _VI_FUNC rshmc804x_ReadDataCatalog(ViSession instrSession,
                                            ViInt32   device,
                                            ViInt32   bufferSize,
                                            ViChar    catalog[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, bufferSize, 0, 2147483647),
            3, "Buffer Size");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, catalog), 4, "Catalog");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "DATA:LIST? %s", deviceArr[device]);
    checkErr(RsCore_QueryViStringUnknownLengthToUserBuffer(instrSession, cmd, bufferSize, catalog, NULL));

    checkErr(rshmc804x_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  List all available files of the selected memory (INT -> Internal Memory, EXT -> USB Stick, DEF -> Internal Memory).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rshmc804x_init or
/// HIPAR instrSession/rshmc804x_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR enabled/Enables / Disables global data logging function.
/// HIPAR mode/Selects the mode of the logging function.
/// HIPAR count/Sets the number of samples in "COUNt" mode.
/// HIPAR time/Sets the time in "TIME" mode.
/// HIPAR interval/Sets the interval between two measurements in seconds.
ViStatus _VI_FUNC rshmc804x_ConfigureLogging(ViSession instrSession,
                                             ViBoolean enabled,
                                             ViInt32   mode,
                                             ViInt32   count,
                                             ViReal64  time,
                                             ViReal64  interval)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rshmc804x_SetAttributeViInt32(instrSession, "", RSHMC804X_ATTR_OUTPUTS_MEAS_LOG_MODE, mode),
            3, "Mode");

    switch (mode)
    {
    case RSHMC804X_VAL_LOGGING_MODE_COUNT:
        viCheckParm(rshmc804x_SetAttributeViInt32(instrSession, "", RSHMC804X_ATTR_OUTPUTS_MEAS_LOG_COUNT, count),
                4, "Count");
        break;
    case RSHMC804X_VAL_LOGGING_MODE_TIME:
        viCheckParm(rshmc804x_SetAttributeViReal64(instrSession, "", RSHMC804X_ATTR_OUTPUTS_MEAS_LOG_TIME, time),
                5, "Time");
        break;
    }

    viCheckParm(rshmc804x_SetAttributeViReal64(instrSession, "", RSHMC804X_ATTR_OUTPUTS_MEAS_LOG_INTERVAL, interval),
            6, "Interval");

    viCheckParm(rshmc804x_SetAttributeViBoolean(instrSession, "", RSHMC804X_ATTR_OUTPUTS_MEAS_LOG_ENABLED, enabled),
            2, "Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Enables / Disables data logging function for the selected channel.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rshmc804x_init or
/// HIPAR instrSession/rshmc804x_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Numerical selection of a channel
/// HIPAR state/Turns the data logging function for the selected channel on or off.
ViStatus _VI_FUNC rshmc804x_ConfigureChannelLoggingState(ViSession instrSession,
                                                         ViInt32   channel,
                                                         ViBoolean state)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rshmc804x_SetAttributeViInt32(instrSession, "", RSHMC804X_ATTR_OUTPUTS_SELECTED_CHANNEL, channel),
            2, "Channel");

    viCheckParm(rshmc804x_SetAttributeViBoolean(instrSession, "", RSHMC804X_ATTR_OUTPUTS_MEAS_CHANNEL_LOG_ENABLED, state),
            3, "State");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Enables (ON/1) / disables (OFF/0) the power counter function.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rshmc804x_init or
/// HIPAR instrSession/rshmc804x_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Numerical selection of a channel
/// HIPAR energyState/Turns the power counter function on or off.
ViStatus _VI_FUNC rshmc804x_ConfigureEnergyState(ViSession instrSession,
                                                 ViInt32   channel,
                                                 ViBoolean energyState)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rshmc804x_SetAttributeViInt32(instrSession, "", RSHMC804X_ATTR_OUTPUTS_SELECTED_CHANNEL, channel),
            2, "Channel");

    viCheckParm(rshmc804x_SetAttributeViBoolean(instrSession, "", RSHMC804X_ATTR_OUTPUTS_MEAS_ENERGY_ENABLED, energyState),
            3, "Energy State");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Enables (On/1) / Disables (Off/0) the power sequencing function.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rshmc804x_init or
/// HIPAR instrSession/rshmc804x_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Numerical selection of a channel
/// HIPAR state/Turns the sequence on or off.
ViStatus _VI_FUNC rshmc804x_ConfigureSequenceState(ViSession instrSession,
                                                   ViInt32   channel,
                                                   ViBoolean state)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rshmc804x_SetAttributeViInt32(instrSession, "", RSHMC804X_ATTR_OUTPUTS_SELECTED_CHANNEL, channel),
            2, "Channel");

    viCheckParm(rshmc804x_SetAttributeViBoolean(instrSession, "", RSHMC804X_ATTR_OUTPUTS_SEQUENCE_ENABLED, state),
            3, "State");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Enables and configures the delay of the selected channel.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rshmc804x_init or
/// HIPAR instrSession/rshmc804x_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Numerical selection of a channel
/// HIPAR enabled/Enables (On/1) / Disables (Off/0) the delay of the selected channel.
/// HIPAR delay/Set the delay in sequencing module for the selected channel.
ViStatus _VI_FUNC rshmc804x_ConfigureSequenceDelay(ViSession instrSession,
                                                   ViInt32   channel,
                                                   ViBoolean enabled,
                                                   ViReal64  delay)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rshmc804x_SetAttributeViInt32(instrSession, "", RSHMC804X_ATTR_OUTPUTS_SELECTED_CHANNEL, channel),
            2, "Channel");

    viCheckParm(rshmc804x_SetAttributeViReal64(instrSession, "", RSHMC804X_ATTR_OUTPUTS_SEQUENCE_DELAY, delay),
            4, "Delay");

    viCheckParm(rshmc804x_SetAttributeViBoolean(instrSession, "", RSHMC804X_ATTR_OUTPUTS_SEQUENCE_DELAY_ENABLED, enabled),
            3, "Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Specifies the type of EXTERNAL trigger input -> POSitive (rising edge), NEGative (falling Edge).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rshmc804x_init or
/// HIPAR instrSession/rshmc804x_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR triggerSlope/Specifies the type.
ViStatus _VI_FUNC rshmc804x_ConfigureTriggerSlope(ViSession instrSession,
                                                  ViInt32   triggerSlope)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rshmc804x_SetAttributeViInt32(instrSession, "", RSHMC804X_ATTR_TRIGGER_SLOPE, triggerSlope),
            3, "Trigger Slope");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function returns external trigger slope.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rshmc804x_init or
/// HIPAR instrSession/rshmc804x_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR triggerSlope/Returns the type of external trigger input.
ViStatus _VI_FUNC rshmc804x_QueryTriggerSlope(ViSession instrSession,
                                              ViInt32*  triggerSlope)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rshmc804x_GetAttributeViInt32(instrSession, "", RSHMC804X_ATTR_TRIGGER_SLOPE, triggerSlope),
            2, "Trigger Slope");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Query if OVP is tripped (1) or is not tripped (0).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rshmc804x_init or
/// HIPAR instrSession/rshmc804x_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Numerical selection of a channel
/// HIPAR tripped/Query if OVP is tripped (1) or is not tripped (0).
ViStatus _VI_FUNC rshmc804x_QueryOVPTripped(ViSession  instrSession,
                                            ViInt32    channel,
                                            ViBoolean* tripped)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rshmc804x_SetAttributeViInt32(instrSession, "", RSHMC804X_ATTR_OUTPUTS_SELECTED_CHANNEL, channel),
            2, "Channel");

    viCheckParm(rshmc804x_GetAttributeViBoolean(instrSession, "", RSHMC804X_ATTR_OUTPUTS_OVP_TRIPPED, tripped),
            3, "Tripped");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Query if OPP is tripped (1) or is not tripped (0).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rshmc804x_init or
/// HIPAR instrSession/rshmc804x_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Numerical selection of a channel
/// HIPAR tripped/Query if OPP is tripped (1) or is not tripped (0).
ViStatus _VI_FUNC rshmc804x_QueryOPPTripped(ViSession  instrSession,
                                            ViInt32    channel,
                                            ViBoolean* tripped)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rshmc804x_SetAttributeViInt32(instrSession, "", RSHMC804X_ATTR_OUTPUTS_SELECTED_CHANNEL, channel),
            2, "Channel");

    viCheckParm(rshmc804x_GetAttributeViBoolean(instrSession, "", RSHMC804X_ATTR_OUTPUTS_OPP_TRIPPED, tripped),
            3, "Tripped");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Query, if the fuse has tripped.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rshmc804x_init or
/// HIPAR instrSession/rshmc804x_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Numerical selection of a channel
/// HIPAR tripped/Query, if the fuse has tripped.
ViStatus _VI_FUNC rshmc804x_QueryFuseTripped(ViSession  instrSession,
                                             ViInt32    channel,
                                             ViBoolean* tripped)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rshmc804x_SetAttributeViInt32(instrSession, "", RSHMC804X_ATTR_OUTPUTS_SELECTED_CHANNEL, channel),
            2, "Channel");

    viCheckParm(rshmc804x_GetAttributeViBoolean(instrSession, "", RSHMC804X_ATTR_OUTPUTS_FUSE_TRIPPED, tripped),
            3, "Tripped");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Enables (On/1) / Disables (Off/0) the trigger input on the rear
/// HIFN  connector for arbitrary functionality.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rshmc804x_init or
/// HIPAR instrSession/rshmc804x_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Numerical selection of a channel
/// HIPAR triggered/Enables (On/1) / Disables (Off/0) the trigger input on the rear
/// HIPAR triggered/connector for arbitrary functionality.
ViStatus _VI_FUNC rshmc804x_QueryArbitraryTriggered(ViSession  instrSession,
                                                    ViInt32    channel,
                                                    ViBoolean* triggered)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rshmc804x_SetAttributeViInt32(instrSession, "", RSHMC804X_ATTR_OUTPUTS_SELECTED_CHANNEL, channel),
            2, "Channel");

    viCheckParm(rshmc804x_GetAttributeViBoolean(instrSession, "", RSHMC804X_ATTR_OUTPUTS_ARBITRARY_TRIGGERED, triggered),
            3, "Tripped");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Enables (On/1) / Disables (Off/0) the trigger function.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rshmc804x_init or
/// HIPAR instrSession/rshmc804x_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Numerical selection of a channel
/// HIPAR triggered/Enables (On/1) / Disables (Off/0) the trigger function.
ViStatus _VI_FUNC rshmc804x_QuerySequenceTriggered(ViSession  instrSession,
                                                   ViInt32    channel,
                                                   ViBoolean* triggered)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rshmc804x_SetAttributeViInt32(instrSession, "", RSHMC804X_ATTR_OUTPUTS_SELECTED_CHANNEL, channel),
            2, "Channel");

    viCheckParm(rshmc804x_GetAttributeViBoolean(instrSession, "", RSHMC804X_ATTR_OUTPUTS_SEQUENCE_TRIGGERED, triggered),
            3, "Tripped");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Enables (On/1) / Disables (Off/0) the trigger function.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rshmc804x_init or
/// HIPAR instrSession/rshmc804x_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR triggered/Enables (On/1) / Disables (Off/0) the trigger function.
ViStatus _VI_FUNC rshmc804x_ConfigureLoggingTriggered(ViSession instrSession,
                                                      ViBoolean triggered)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rshmc804x_SetAttributeViBoolean(instrSession, "", RSHMC804X_ATTR_OUTPUTS_MEAS_LOG_TRIGGERED, triggered),
            2, "Triggered");

Error:
    (void)RsCore_UnlockSession(instrSession);

    return error;
}

/// HIFN  Enables (On/1) / Disables (Off/0) the trigger function.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rshmc804x_init or
/// HIPAR instrSession/rshmc804x_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR triggered/Enables (On/1) / Disables (Off/0) the trigger function.
ViStatus _VI_FUNC rshmc804x_QueryLoggingTriggered(ViSession  instrSession,
                                                  ViBoolean* triggered)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rshmc804x_GetAttributeViBoolean(instrSession, "", RSHMC804X_ATTR_OUTPUTS_MEAS_LOG_TRIGGERED, triggered),
            2, "Triggered");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Resets the state of the OVP function.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rshmc804x_init or
/// HIPAR instrSession/rshmc804x_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Numerical selection of a channel
ViStatus _VI_FUNC rshmc804x_ResetOutputProtection(ViSession instrSession,
                                                  ViInt32   channel)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rshmc804x_SetAttributeViInt32(instrSession, "", RSHMC804X_ATTR_OUTPUTS_SELECTED_CHANNEL, channel),
            2, "Channel");

    checkErr(rshmc804x_SetAttributeViString(instrSession, "", RSHMC804X_ATTR_OUTPUTS_RESET_OUTPUT_PROTECTION, NULL));

    checkErr(rshmc804x_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Resets the state of the OPP function.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rshmc804x_init or
/// HIPAR instrSession/rshmc804x_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Numerical selection of a channel
ViStatus _VI_FUNC rshmc804x_ResetPowerProtection(ViSession instrSession,
                                                 ViInt32   channel)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rshmc804x_SetAttributeViInt32(instrSession, "", RSHMC804X_ATTR_OUTPUTS_SELECTED_CHANNEL, channel),
            2, "Channel");

    checkErr(rshmc804x_SetAttributeViString(instrSession, "", RSHMC804X_ATTR_OUTPUTS_RESET_POWER_PROTECTION, NULL));

    checkErr(rshmc804x_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Resets the amount of the energie counter function.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rshmc804x_init or
/// HIPAR instrSession/rshmc804x_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Numerical selection of a channel
ViStatus _VI_FUNC rshmc804x_ResetEnergyCounter(ViSession instrSession,
                                               ViInt32   channel)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rshmc804x_SetAttributeViInt32(instrSession, "", RSHMC804X_ATTR_OUTPUTS_SELECTED_CHANNEL, channel),
            2, "Channel");

    checkErr(rshmc804x_SetAttributeViString(instrSession, "", RSHMC804X_ATTR_OUTPUTS_MEAS_ENERGY_RESET, NULL));

    checkErr(rshmc804x_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Deletes the arbitrary waveform values.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rshmc804x_init or
/// HIPAR instrSession/rshmc804x_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Numerical selection of a channel
ViStatus _VI_FUNC rshmc804x_ResetArbitrary(ViSession instrSession,
                                           ViInt32   channel)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rshmc804x_SetAttributeViInt32(instrSession, "", RSHMC804X_ATTR_OUTPUTS_SELECTED_CHANNEL, channel),
            2, "Channel");

    checkErr(rshmc804x_SetAttributeViString(instrSession, "", RSHMC804X_ATTR_OUTPUTS_ARBITRARY_RESET, NULL));

    checkErr(rshmc804x_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Resets the incremental step size for the voltage or current.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rshmc804x_init or
/// HIPAR instrSession/rshmc804x_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Numerical selection of a channel
/// HIPAR stepSizeType/The kind of step size to be configured.
ViStatus _VI_FUNC rshmc804x_ResetStepSize(ViSession instrSession,
                                          ViInt32   channel,
                                          ViInt32   stepSizeType)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, stepSizeType, RSHMC804X_VAL_STEP_SIZE_VOLTAGE, RSHMC804X_VAL_STEP_SIZE_CURRENT),
            3, "Step Size Type");

    viCheckParm(rshmc804x_SetAttributeViInt32(instrSession, "", RSHMC804X_ATTR_OUTPUTS_SELECTED_CHANNEL, channel),
            2, "Channel");

    switch (stepSizeType)
    {
    case RSHMC804X_VAL_STEP_SIZE_VOLTAGE:
        checkErr(rshmc804x_SetAttributeViString(instrSession, "", RSHMC804X_ATTR_OUTPUTS_RESET_VOLTAGE_STEP_SIZE, NULL));
        break;
    case RSHMC804X_VAL_STEP_SIZE_CURRENT:
        checkErr(rshmc804x_SetAttributeViString(instrSession, "", RSHMC804X_ATTR_OUTPUTS_RESET_CURRENT_STEP_SIZE, NULL));
        break;
    }

    checkErr(rshmc804x_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Takes a measurement on the output signal and returns the measured value.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rshmc804x_init or
/// HIPAR instrSession/rshmc804x_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Numerical selection of a channel
/// HIPAR measurementType/The type of measurement to take.
/// HIPAR measurement/The measured value.
ViStatus _VI_FUNC rshmc804x_Measure(ViSession instrSession,
                                    ViInt32   channel,
                                    ViInt32   measurementType,
                                    ViReal64* measurement)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, measurementType, RSHMC804X_VAL_MEASURE_VOLTAGE, RSHMC804X_VAL_MEASURE_ENERGY),
            3, "Measurement Type");

    viCheckParm(rshmc804x_SetAttributeViInt32(instrSession, "", RSHMC804X_ATTR_OUTPUTS_SELECTED_CHANNEL, channel),
            2, "Channel");

    switch (measurementType)
    {
    case RSHMC804X_VAL_MEASURE_VOLTAGE:
        strcpy(cmd, "MEAS?");
        break;
    case RSHMC804X_VAL_MEASURE_CURRENT:
        strcpy(cmd, "MEAS:CURR?");
        break;
    case RSHMC804X_VAL_MEASURE_POWER:
        strcpy(cmd, "MEAS:POW?");
        break;
    case RSHMC804X_VAL_MEASURE_ENERGY:
        strcpy(cmd, "MEAS:ENER?");
        break;
    }

    checkErr(rshmc804x_QueryViReal64(instrSession, cmd, measurement));
    checkErr(rshmc804x_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN If TRUE (default after init is FALSE), the driver calls SYST:ERR? automatically in CheckStatus() when Instrument Status Error is detected.
/// HIFN Use the function rshmc804x_GetError afterwards to get the detailed information about the instrument error(s).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rshmc804x_init or
/// HIPAR instrSession/rshmc804x_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR autoSystErrQuery/If TRUE (default after init is FALSE), the driver calls SYST:ERR? automatically in CheckStatus() when Instrument Status Error is detected.
ViStatus _VI_FUNC rshmc804x_ConfigureAutoSystemErrQuery(ViSession instrSession,
                                                        ViBoolean autoSystErrQuery)
{
    ViStatus         error = VI_SUCCESS;
    RsCoreSessionPtr rsSession = NULL;

    checkErr(RsCore_GetRsSession(instrSession, &rsSession));
    rsSession->autoSystErrQuery = autoSystErrQuery;

Error:
    return error;
}

/// HIFN Session-related mutex switch.
/// HIFN If TRUE (default after init is FALSE), the driver sets a mutual-exclusion mechanism for every driver's hi-level function and attribute access.
/// HIFN Set this value to TRUE if you plan to use one session in more than one thread.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rshmc804x_init or
/// HIPAR instrSession/rshmc804x_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR multiThreadLocking/Session-related mutex switch
ViStatus _VI_FUNC rshmc804x_ConfigureMultiThreadLocking(ViSession instrSession,
                                                        ViBoolean multiThreadLocking)
{
    ViStatus         error = VI_SUCCESS;
    RsCoreSessionPtr rsSession = NULL;

    checkErr(RsCore_GetRsSession(instrSession, &rsSession));
    rsSession->multiThreadLocking = multiThreadLocking;

Error:
    return error;
}
