/*****************************************************************************
 *  Rohde&Schwarz DC power supplies instrument driver
 *  Callbacks
 *
 *  Original Release: May 2014
 *  By: Lukas Hynek
 *
 *  Should you have any technical questions please contact the hotline of
 *  Rohde & Schwarz Vertriebs-GmbH Rohde & Schwarz Support Center
 *
 *  e-mail: CustomerSupport@rohde-schwarz.com
 *

 *****************************************************************************/

#include "rshmc804x.h"

#if (defined(WIN32) || defined(_WIN32) || defined(__WIN32__) || defined(__NT__))
#define ANSI
#endif
#ifndef ANSI /* UNIX compatible */
#include <sys/timeb.h>
#include <sys/types.h>
#include <unistd.h>
#define Sleep sleep
#else /* ANSI compatible */
#include <windows.h>
#endif

#pragma warning(disable : 4100) // unreferenced formal parameter

/*****************************************************************************
 *------------------------ Global Session Callbacks -------------------------*
 *****************************************************************************/

/*****************************************************************************
 *- Read Callbacks ----------------------------------------------------------*
 *****************************************************************************/

/*****************************************************************************
 * Function: rshmc804x_empty_ReadCallback
 * Purpose:  This function overrides standard calback funtion. Returns value
 *           from cache
 *****************************************************************************/
ViStatus rshmc804x_empty_ReadCallback(ViSession          instrSession,
                                      ViConstString      repCapName,
                                      RsCoreAttributePtr attr)
{
    return VI_SUCCESS;
}

/*****************************************************************************
 * Function: rshmc804x_logging_ReadCallback
 * Purpose:  This function overrides standard calback funtion. Returns value
 *           from cache
 *****************************************************************************/
ViStatus rshmc804x_logging_ReadCallback(ViSession          instrSession,
                                        ViConstString      repCapName,
                                        RsCoreAttributePtr attr)
{
    return VI_SUCCESS;
}

/*****************************************************************************
 * Function: rshmc804x_MasterEnabled_ReadCallback
 * Purpose:  NGE has a different command
 *
 *****************************************************************************/
ViStatus rshmc804x_MasterEnabled_ReadCallback(ViSession          instrSession,
                                              ViConstString      repCapName,
                                              RsCoreAttributePtr attr)
{
    ViStatus  error = VI_SUCCESS;
    ViBoolean output;

    if (!rshmc804x_IsInstrumentNgxFamily(instrSession))
    {
        // Standard read callback for a non-NGE instrument
        return RsCore_ReadCallback(instrSession, repCapName, attr);
    }

    // If no command available, exit the function
    if (isNullOrEmpty(attr->command))
        goto Error;

    if (attr->dataType != RS_VAL_BOOLEAN)
        checkErr(RsCore_GenerateInvalidDataTypeError(instrSession, attr->dataType, NULL));

    checkErr(RsCore_QueryViBoolean(instrSession, "OUTP:GEN?", &output));
    *((ViPBoolean)attr->p2value) = output;

Error:
    return error;
}

/*****************************************************************************
 * Function: rshmc804x_ChannelOnlyEnabled_ReadCallback
 * Purpose:  NGE has a different command
 *
 *****************************************************************************/
ViStatus rshmc804x_ChannelOnlyEnabled_ReadCallback(ViSession          instrSession,
                                                   ViConstString      repCapName,
                                                   RsCoreAttributePtr attr)
{
    ViStatus  error = VI_SUCCESS;
    ViBoolean output;

    if (!rshmc804x_IsInstrumentNgxFamily(instrSession))
    {
        // Standard read callback for a non-NGE instrument
        return RsCore_ReadCallback(instrSession, repCapName, attr);
    }

    // If no command available, exit the function
    if (isNullOrEmpty(attr->command))
        goto Error;

    if (attr->dataType != RS_VAL_BOOLEAN)
        checkErr(RsCore_GenerateInvalidDataTypeError(instrSession, attr->dataType, NULL));

    checkErr(RsCore_QueryViBoolean(instrSession, "OUTP:SEL?", &output));
    *((ViPBoolean)attr->p2value) = output;

Error:
    return error;
}

/*****************************************************************************
 *- Write Callbacks ---------------------------------------------------------*
 *****************************************************************************/

/*****************************************************************************
 * Function: rshmc804x_logging_WriteCallback
 * Purpose:  This function overrides standard calback funtion.
 *****************************************************************************/
ViStatus rshmc804x_logging_WriteCallback(ViSession          instrSession,
                                         ViConstString      repCapName,
                                         RsCoreAttributePtr attr,
                                         void*              value)
{
    return VI_SUCCESS;
}

/*****************************************************************************
 * Function: rshmc804x_empty_WriteCallback
 * Purpose:  This function overrides standard calback funtion. Does not write
 *           anything to instrument, just updates the cache
 *****************************************************************************/
ViStatus rshmc804x_empty_WriteCallback(ViSession          instrSession,
                                       ViConstString      repCapName,
                                       RsCoreAttributePtr attr,
                                       void*              value)
{
    return VI_SUCCESS;
}

/*****************************************************************************
 * Function: rshmc804x_MasterEnabled_WriteCallback
 * Purpose:  NGE needs different command + Delay if ON
 *
 *****************************************************************************/
ViStatus rshmc804x_MasterEnabled_WriteCallback(ViSession          instrSession,
                                               ViConstString      repCapName,
                                               RsCoreAttributePtr attr,
                                               void*              value)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    if (!rshmc804x_IsInstrumentNgxFamily(instrSession))
    {
        // Standard read callback for a non-NGE instrument
        return RsCore_WriteCallback(instrSession, repCapName, attr, value);
    }

    // If no command available, exit the function
    if (isNullOrEmpty(attr->command))
        goto Error;

    if (attr->dataType != RS_VAL_BOOLEAN)
        checkErr(RsCore_GenerateInvalidDataTypeError(instrSession, attr->dataType, NULL));

    ViBoolean newValue = *(ViBoolean *)value;
    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "OUTP:GEN %s;*OPC?", newValue ? "ON" : "OFF");
    checkErr(RsCore_QueryViBoolean(instrSession, cmd, NULL));
    if (newValue == VI_TRUE)
    {
        Sleep(500); // NGE needs cca 500 ms to switch the output ON
    }

Error:
    return error;
}

/*****************************************************************************
 * Function: rshmc804x_ChannelOnlyEnabled_WriteCallback
 * Purpose:  NGE needs different command + delay if Master Out is ON.
*            *OPC? is added at the end for all instruments
 *
 *****************************************************************************/
ViStatus rshmc804x_ChannelOnlyEnabled_WriteCallback(ViSession          instrSession,
                                                    ViConstString      repCapName,
                                                    RsCoreAttributePtr attr,
                                                    void*              value)
{
    ViStatus  error = VI_SUCCESS;
    ViChar    cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViChar    *buffer = NULL;
    ViBoolean newValue = *(ViBoolean *)value;
    ViBoolean masterOut;

    // If no command available, exit the function
    if (isNullOrEmpty(attr->command))
        goto Error;

    if (attr->dataType != RS_VAL_BOOLEAN)
        checkErr(RsCore_GenerateInvalidDataTypeError(instrSession, attr->dataType, NULL));

    // If NGE, use the direct command
    if (rshmc804x_IsInstrumentNgxFamily(instrSession))
    {
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "OUTP:SEL %s;*OPC?", newValue ? "ON" : "OFF");
        checkErr(RsCore_QueryViBoolean(instrSession, cmd, NULL));

        //Only apply delay if the master output is ON
        checkErr(RsCore_QueryViBoolean(instrSession, "OUTP:GEN?", &masterOut));

        if(newValue == VI_TRUE && masterOut == VI_TRUE)
        {
            Sleep(500); //Delay 500 ms if the new status is ON and the Master is ON
        }
    }
    else // For HMC804x, use the attribute command, but with the *OPC? query at the end
    {
        checkErr(RsCore_BuildAttrCommandString(instrSession, attr, repCapName, &buffer));
        checkErr(RsCore_StrCatDynamicString(instrSession, &buffer, " ", newValue ? "ON" : "OFF", ";*OPC?"));
        checkErr(RsCore_QueryViBoolean(instrSession, cmd, NULL));
    }

Error:
    if (buffer)
        free(buffer);

    return error;
}

/*****************************************************************************
 * Function: rshmc804x_SelectChannel_WriteCallback
 * Purpose:  Compare desired value with Chache value, only send to the instrument (plus *OPC?) if they are different
 *
 *****************************************************************************/
ViStatus rshmc804x_SelectChannel_WriteCallback(ViSession          instrSession,
                                               ViConstString      repCapName,
                                               RsCoreAttributePtr attr,
                                               void*              value)
{
    ViStatus error = VI_SUCCESS;
    ViChar   *cmd = NULL;
    ViChar   cmdComplete[RS_MAX_MESSAGE_BUF_SIZE];
    ViInt32  newValue = *(ViInt32 *)value;
    ViInt32  currentValue;

    // If no command available, exit the function
    if (isNullOrEmpty(attr->command))
        goto Error;

    if (attr->dataType != RS_VAL_INT32)
        checkErr(RsCore_GenerateInvalidDataTypeError(instrSession, attr->dataType, NULL));

    currentValue = (*(ViInt32 *)attr->p2value);

    if (newValue != currentValue)
    {
        checkErr(RsCore_BuildAttrCommandString(instrSession, attr, repCapName, &cmd));
        snprintf(cmdComplete, RS_MAX_MESSAGE_BUF_SIZE, "%s %ld;*OPC?", cmd, newValue);
        checkErr(RsCore_QueryViBoolean(instrSession, cmdComplete, NULL));
    }

Error:
    if (cmd)
        free(cmd);

    return error;
}


/*****************************************************************************
 * Function: rshmc804x_OutputEnabled_WriteCallback
 * Purpose:  NGE needs + delay if switching ON. *OPC? is added at the end
 *
 *****************************************************************************/
ViStatus rshmc804x_OutputEnabled_WriteCallback(ViSession          instrSession,
                                               ViConstString      repCapName,
                                               RsCoreAttributePtr attr,
                                               void*              value)
{
    ViStatus  error = VI_SUCCESS;
    ViChar    *cmd = NULL;
    ViBoolean newValue = *(ViBoolean *)value;

    // If no command available, exit the function
    if (isNullOrEmpty(attr->command))
        goto Error;

    if (attr->dataType != RS_VAL_BOOLEAN)
        checkErr(RsCore_GenerateInvalidDataTypeError(instrSession, attr->dataType, NULL));

    checkErr(RsCore_BuildAttrCommandString(instrSession, attr, repCapName, &cmd));
    checkErr(RsCore_StrCatDynamicString(instrSession, &cmd, " ", newValue ? "ON" : "OFF", ";*OPC?"));
    checkErr(RsCore_QueryViBoolean(instrSession, cmd, NULL));

    if (rshmc804x_IsInstrumentNgxFamily(instrSession))
    {
        Sleep(500); //For NGE: Delay 500 ms
    }

Error:
    if (cmd)
        free(cmd);

    return error;
}